@@ -2,7 +2,7 @@
 #
 # Version
 #      $Source: C:/src/perl/File/Repl/RCS/Repl.pm $
-#      $Revision: 2.1 $
+#      $Revision: 2.3 $
 #      $State: Exp $
 #
 # Start comments/code here - will not be processed into manual pages
@@ -11,6 +11,12 @@
 #
 # Revision history:
 #      $Log: Repl.pm $
+#      Revision 2.3  2015/11/03 18:07:45  Dave.Roberts
+#      interim release - corrections to help manage situations when a file and directory are compared
+#
+#      Revision 2.2  2015/11/01 22:36:38  Dave.Roberts
+#      removed Win32::Admin requirement from documentation
+#
 #      Revision 2.1  2015/07/15 20:51:29  Dave.Roberts
 #      added timestamp info for reading directories
 #
@@ -153,8 +159,6 @@ if ($^O eq 'MSWin32') {        # is this a win32 system ?
     my($string,$major,$minor,$build,$id) = Win32::GetOSVersion();
     if ( $id == 2 ) {   #  Machine is NT (0=Win32s, 1=Win9x etc)
       eval "use Win32::API";
-#      eval "use Win32::AdminMisc";
-
       my $lpTimeZoneInformation = "\0" x 172;   # space for struct _TIME_ZONE_INFORMATION
       my $GetTimeZoneInformation = new Win32::API("kernel32", 'GetTimeZoneInformation', ['P'], 'N');
       croak "\n ERROR: failed to import GetTimeZoneInformation API function\n" if !$GetTimeZoneInformation;
@@ -162,7 +166,7 @@ if ($^O eq 'MSWin32') {        # is this a win32 system ?
       croak "\n ERROR: GetTimeZoneInformation returned invalid data: " . Win32::FormatMessage(Win32::GetLastError())
       if $ISDST == TIME_ZONE_ID_INVALID;
       my ($Bias,$StandardBias,$DaylightBias) = unpack "l x80 l x80 l", $lpTimeZoneInformation;
-
+      
 # $ISDST == 0 -  No Daylight Savings in this timezone (no transition dates defined for this tz)
 # $ISDST == 1 -  Standard time
 # $ISDST == 2 -  Daylight Savings time
@@ -190,7 +194,7 @@ our @EXPORT = qw(
 
 );
 
-our $VERSION = sprintf("%d.%d", q$Revision: 2.1 $ =~ /(\d+)\.(\d+)/);
+our $VERSION = sprintf("%d.%d", q$Revision: 2.3 $ =~ /(\d+)\.(\d+)/);
 
 # Preloaded methods go here.
 #---------------------------------------------------------------------
@@ -448,6 +452,7 @@ if ($caller eq "Update") {
       print "Illegal display option called ($disp)\n";
       return 0;
     }
+    # so we are always copying A to B
     $msg = " $a $disp $b";
     if ( $amtime == 0 ) {
       $age = $bmtime;
@@ -475,9 +480,13 @@ if ($caller eq "Update") {
       print "$msg\n"  if ($verbose >= 1);
       return TRUE;
     }
-    return FALSE unless &$md(dirname($B)); # Make sure the parent of the target file exists
+    unless (&$md(dirname($B))){ # Make sure the parent of the target file exists
+       print "parent of target $B does not exist\n";
+       printf "( %s )\n",dirname($B);
+       return FALSE;
+     }
     if ( -f $A ) {
-      if ( -f $B ) {
+      if ( -f $B ) {  # create a backup copy of target file $B
         $Btmp = $B . '.X';
         while ( -f $Btmp ) {  #  Find a temporary file name to copy target to (allows rollback after a copy failure)
           $Btmp .= 'X';
@@ -488,6 +497,10 @@ if ($caller eq "Update") {
           undef $Btmp;
         }
       }
+      if ( -d $B ){ # if the target file is a directory.... then remove it
+        print " *************** removing directory tree $B to allow file copy\n";  
+        &deltree2($B);
+        }
 
       if ( copy ($A,$B) ) {
         print "$msg\n"  if ($verbose >= 1);
@@ -510,8 +523,14 @@ if ($caller eq "Update") {
           }
         }
       }
-      }else{
-      if ( ! -d $B ) {
+      }else{ # $A is a directory...
+        if ( -f $B ){
+        # if the target is a file - and needs to be a directory.... then remove it
+        print " *************** removing file $B to allow replacement as a directory\n";  
+       unlink $B || carp "Failed to delete file $B\n";
+        }
+        if ( ! -d $B ) {
+
         mkdir($B,0777) && return TRUE || carp "Unable to create directory $B\n";
         print "$msg - (new directory)\n"  if ($verbose >= 1);
       }
@@ -578,9 +597,15 @@ if ($caller eq "Update") {
 
 #****************************************************************
 # sub to delete directory trees
+# $targ    directory name
+# $dir     path to directory
+# $reftime reference to time hash (to allow record deletion)
+# $reftype reference to type hash (to allow record deletion)
+# $top      set true to remove top level directory
 #****************************************************************
 $deltree = sub {
   my($targ,$dir,$reftime,$reftype,$top) = @_;
+
   my $xxx = sub {
     ($tmp = $File::Find::name) =~ s/^\Q$dir//; # identify relative filename (to $dir)
     return if ($tmp eq $targ);                 # Don't remove top level directory unless $top == TRUE
@@ -592,7 +617,7 @@ $deltree = sub {
   finddepth(\&$xxx, $dir . $targ);
   if ($top) {
     chdir $dir;
-		 if (rmdir "$dir$targ") {
+    if (rmdir "$dir$targ") {
       delete $reftime->{$targ};
       delete $reftype->{$targ};
     }else{
@@ -760,6 +785,34 @@ if ( $caller eq "Update" ) {
     $afile  = $r_con->{dira} . $aName;
     $bfile  = $r_con->{dirb} . $bName;
 
+# if btype is a file, and atype a directory, and we write a>b then ignore time rules
+# - we will remove the directory
+  if (($btype & 0x01) & !($atype & 0x01)&( $mode =~ /^(a>b)|(a<>b)|(A>B)|(A>B!)$/ )) {
+#        printf "AFILE %s %s [%s]\n", $afile,$atype,($atype & 0x01);
+#        printf "BFILE %s %s [%s]\n", $bfile,$btype,($btype & 0x01);
+        $refb->{$bName} = $amtime if (&$fc($afile,$bfile,$amtime,0,"-->",$atype));
+        # also need logic here to remove entries for any files in $bfile directory structure
+        # from $refb, $btype etc
+      }
+
+# if atype is a file, and btype a directory, and we write a<b then ignore time rules
+# - we will remove the directory
+  if (($atype & 0x01) & !($btype & 0x01)&( $mode =~ /^(a<b)|(a<>b)|(A<B)|(A<B!)$/ )) {
+#        printf "AFILE %s %s [%s]\n", $afile,$atype,($atype & 0x01);
+#        printf "BFILE %s %s [%s]\n", $bfile,$btype,($btype & 0x01);
+        $refa->{$aName} = $bmtime if (&$fc($afile,$bfile,$amtime,$bmtime,"<--",$btype));
+        # also need logic here to remove entries for any files in $afile directory structure
+        # from $refa, $atype etc
+      }
+
+if ((($atype & 0x01) != ($btype & 0x01))&( $mode =~ /^(A<>B)$/ )) {
+#        printf "AFILE %s %s [%s]\n", $afile,$atype,($atype & 0x01);
+#        printf "BFILE %s %s [%s]\n", $bfile,$btype,($btype & 0x01);
+         printf "objects %s and %s are different types, mode does not allow an action to be taken\n",$afile, $bfile;
+}
+
+
+
 # Skip directories as their time can't be set (NH251100)
     next if -d $afile;
 
@@ -950,6 +1003,30 @@ sub elapsed{
   my($elapsed) = $start_time - time;
   return $elapsed;
 }
+#=====================================================================
+sub deltree2{
+  my($root)=@_;
+  my(@dirlist);
+  sub finddir {
+    if ( -d ) {
+		push (@dirlist,$File::Find::name);
+    }
+  }
+  sub findfile {
+  if ( -f ) {
+     print " -   unable to delete $File::Find::name \n" if (! unlink $File::Find::name );
+  }
+}
+  find(\&findfile,$root);
+  find(\&finddir,$root);
+    while ( @dirlist ) {
+    my $dir = pop(@dirlist);
+    print "   -   unable to remove $dir\n" if (! rmdir($dir) );
+  }
+    print "   -   unable to remove $root\n" if (! rmdir($root) );
+  
+}
+#========================================================================
 1;
 __END__
 
@@ -1276,7 +1353,6 @@ The Version method returns the File::Repl module version. No calling argument is
   File::Copy;
   File::Basename;
 
-  Win32::AdminMisc (Win32 platforms only)
   Win32::API       (Win32 platforms only)
 
 =head1 TIMEZONE AND FILESYSTEMS
@@ -1471,6 +1547,12 @@ out of the use of the script.
 =head1 CHANGE HISTORY
 
 $Log: Repl.pm $
+Revision 2.3  2015/11/03 18:07:45  Dave.Roberts
+interim release - corrections to help manage situations when a file and directory are compared
+
+Revision 2.2  2015/11/01 22:36:38  Dave.Roberts
+removed Win32::Admin requirement from documentation
+
 Revision 2.1  2015/07/15 20:51:29  Dave.Roberts
 added timestamp info for reading directories
 
