@@ -5,6 +5,7 @@
 
 # Change 1..1 below to 1..last_test_to_print .
 # (It may become useful if the test is moved to ./t subdirectory.)
+use lib 'blib/lib';
 use File::HomeDir;
 BEGIN { $| = 1; print "1..16\n"; }
 my($testno);
@@ -278,7 +279,6 @@ $atime,$mtime,$ctime2,$blksize2,$blocks2)
 print "\n\treplace destination file $tstfileb with a directory of the same name" if $debug;
 $mtime = $mtime -10;
 #print " $mtime \n" if $debug;
-utime ($atime,$mtime, $tstfileb);
 unlink($tstfileb) || die "   -   unable to remove $tstfileb ($!)\n";
 mkdir($tstfileb) || die "   -   unable to create directory $tstfileb ($!)\n";
 utime ($atime,$mtime, $tstfileb);
@@ -313,7 +313,7 @@ print "\n\tok $testno - Update(a>b) with commit=1 method tested - remove an olde
 #   Test the a>b option - with a newer destination directory (removes it and replaces it with a file)
 $testno++;
 undef $ok;
-print "\ntest $testno:--------------------------------------------------------------------------\n";
+print "\ntest $testno:XXX--------------------------------------------------------------------------\n";
 print "\t - Update(a>b) with commit=1 method tested - with a newer destination directory (removes it and replaces it with a file)\n";
 @afiles = @files;
 $tstfile = pop @afiles;
@@ -327,11 +327,11 @@ $atime,$mtime,$ctime2,$blksize2,$blocks2)
 print "\n\treplace destination file $tstfileb with a directory of the same name" if $debug;
 $mtime = $mtime +10;
 #print " $mtime \n" if $debug;
-utime ($atime,$mtime, $tstfileb);
 unlink($tstfileb) || die "   -   unable to remove $tstfileb ($!)\n";
 mkdir($tstfileb) || die "   -   unable to create directory $tstfileb ($!)\n";
 utime ($atime,$mtime, $tstfileb);
 
+
 $hashref = {
   'dira',      $dira,
   'dirb',      $dirb,
@@ -351,11 +351,17 @@ TESTF: foreach (@files) {
       $ok = 1;
       last TESTF;
       }else{
-        print "\$result $result\n";
+        print "\$result $result ($_)\n";
     }
   }
 }
 exit unless ( $ok eq 1 );
+
+# restore original test dir structure
+
+
+
+
 print "\n\tok $testno - Update(a>b) with commit=1 method tested - with a newer destination directory (removes it and replaces it with a file)\n";
 # #####################################################################
 #   Test the a>b option does replicate a file when the destination is older
@@ -363,7 +369,7 @@ print "\n\tok $testno - Update(a>b) with commit=1 method tested - with a newer d
 #   older than the original in $targeta, is replicated succesfully
 $testno++;
 undef $ok;
-print "\ntest $testno:--------------------------------------------------------------------------\n";
+print "\ntest $testno:---------------------------------------------------------------------------\n";
 print "\t - Update(a>b) with commit=1 method tested - does overwrite older file\n";
 my($dev2,$ino2,$mode2,$nlink2,$uid2,$gid2,$rdev2,$size2,
 $atime,$mtime,$ctime2,$blksize2,$blocks2)
@@ -554,7 +560,11 @@ sub _s4 {
     return 2;
   }
   if(-f $file1 ne -f $file2){
-    print "   objetcts $file1 and $file2 are different types\n" if $debug;
+    if ($debug){
+    print "   objects $file1 and $file2 are different types\n";
+    printf "\t%s %s\n",&type($file1),$file1;
+    printf "\t%s %s\n",&type($file2),$file2;
+  }
     return 6;   
   }elsif (compare($file1,$file2) != 0) {
     print "   files $file1 and $file2 are different\n" if $debug;
@@ -580,3 +590,13 @@ sub _s4 {
   }
   return 0;
 }
+sub type {
+  my $object = shift;
+  if(-f $object){
+    return "file";
+  }elsif(-d $object){
+    return "directory";
+  }else{
+    return "unknown";
+  }
+}
